/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: carol@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: DefaultPolicyStrategyLoader.java 1840 2008-06-10 16:26:18Z eyindanga $
 * --------------------------------------------------------------------------
 */
/**
 * Loaders are responsible of loading classes from an archive previously shared in the cluster.
 * This is the default loader.
 */
package org.ow2.cmi.osgi.loader;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.ow2.cmi.config.CMIConfig;
import org.ow2.cmi.controller.server.IPolicyStrategyManager;
import org.ow2.cmi.lb.loader.ArchiveId;
import org.ow2.cmi.lb.loader.ArchiveWrapper;
import org.ow2.cmi.loader.IArchiveWrapper;
import org.ow2.cmi.loader.IDefaultPolicyStrategyLoader;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * The default policy/strategy Loader.
 *
 * @author The new CMI team
 *
 */
public class DefaultPolicyStrategyLoader implements IDefaultPolicyStrategyLoader {

    /**
     * Map of deployed bundles.
     */
    private HashMap<String, String> deployedBundles = new HashMap<String, String>();

    /**
     * The cmi server to interact with.
     */
    private static IPolicyStrategyManager policyStrategyManager = null;

    /**
     * The bundle context.
     */
    private static BundleContext bundleContext;

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory
            .getLog(DefaultPolicyStrategyLoader.class);

    /**
     * Default constructor.
     */
    public DefaultPolicyStrategyLoader() {

    }

    /**
     * Load Load balancing policies/archives from the given <code>archive</code>
     * wrapper.
     *
     * @param archive
     *            The archive wrapper
     */
    public void loadArchive(final IArchiveWrapper archive) {
        LOGGER.debug(" Trying to load classes from archive with id : "
                + archive.getArchiveId());
        ArchiveWrapper arch = (ArchiveWrapper) archive;
        LOGGER
                .debug("Getting new archive for load balancing policies/strategies, previously broadcast in the cluster ");
        ArchiveId archiveId = LoaderUtil
                .buildArchiveIdFromFormattedId((String) arch.getArchiveId());
        LOGGER.debug("Archive host:{0} archive location:{1}", archiveId
                .getHost(), archiveId.getName());
        if (((ArchiveWrapper) archive).toBeInstalled()) {
            InputStream is = null;
            File archiveTmp = null;
            try {
                archiveTmp = LoaderUtil.createTempJarFile(archiveId.getName(),
                        archive.getData());
                is = archiveTmp.toURL().openStream();
                String deployedLocation = CMIConfig.getDeployDirectory()
                        + archiveTmp.getName();
                Bundle archiveBundle = bundleContext.installBundle(
                        deployedLocation, is);
                deployedBundles.put(archiveId.toString(), deployedLocation);
            } catch (Exception e) {
                LOGGER
                        .debug(
                                "Unable to install policy/strategy bundle with Id:{0}, name:{1} ",
                                archiveId.getHost(), archiveId.getName());
            } finally {
                if (archiveTmp != null) {
                    archiveTmp.delete();
                }
                if (is != null) {
                    try {
                        is.close();
                    } catch (IOException e) {
                        LOGGER
                                .error("Unable to close input stream for load balancing archive");
                    }
                }

            }
        }
        LOGGER.debug("archive successfully loaded");
    }

    /**
     * @return the policyStrategyManager
     */
    public static IPolicyStrategyManager getPolicyStrategyManager() {
        return policyStrategyManager;
    }

    /**
     * Sets policy/strategy manager.
     *
     * @param poClusterViewManager
     *            the policyStrategyManager to set
     */
    public void setPolicyStrategyManager(
            final IPolicyStrategyManager poClusterViewManager) {
        policyStrategyManager = poClusterViewManager;
    }

    /**
     * Removes load balancing policies/strategies previously added from the
     * given <code>archive</code> wrapper.
     *
     * @param archive
     *            The archive wrapper
     */
    public void unLoadClasses(final IArchiveWrapper archive) {
    }

    /**
     * Removes load balancing policies/strategies previously added from the
     * given <code>archive</code> wrapper.
     *
     * @param archiveId
     *            The archive Id
     */
    public void unLoadArchive(final Object archiveId) {
        String archiveLocation = deployedBundles.get(archiveId);
        LOGGER.debug("Removing bundle with Id '" + archiveId
                + "' installed at : '" + archiveLocation + "' ");
        File f = new File(archiveLocation);
        if (f.delete()) {
            LOGGER.debug("The bundle was successfully removed");
        }
        deployedBundles.remove(archiveId);
    }

    /**
     * @return the bundleContext
     */
    public static BundleContext getBundleContext() {
        return bundleContext;
    }

    /**
     * Set the budnle context.
     * @param bc
     *            the bundleContext to set
     */
    public static void setBundleContext(final BundleContext bc) {
        bundleContext = bc;
    }

    public void loadArchive(String deployDir, IArchiveWrapper archive) {
        // TODO Auto-generated method stub

    }

}
